<?php

/* Log system manager.
 * Here we get all the logging or debug sentences
 *
 */

class LogService
{
	private $cLog = array();
	private $cFileOutput;
	private $cFilePointer;

	function __construct($siteInstance)
	{
		$this->cFileOutput = FALSE;
		if (Config("log_file") == TRUE) {
			$this->cFileOutput = TRUE;

			$openString = "";
			switch(Config("log_mode"))
			{
				case "append":
					$openString = "a+";
					break;
				case "rewrite":
					$openString = "w";
					break;
				default:
					ERROR("Unknown file logging mode: check the configuration",
						$this->cSiteInstance);
					break;
			}
			$this->cFilePointer = fopen(Config("site_path") . "/" . Config("log_path"), $openString) OR
				ERROR("Unable to open log file for writing: check permissions", $this->cSiteInstance);
		}

		if ($this->cFileOutput == TRUE) {
			$string = "enabled, file: " . Config("site_path") . "/" . Config("log_path");
		} else {
			$string = "disabled";
		}

		$this->addLogMsg(SILENT, "LOG: Log system initialized (file output $string)");
	}


	public
	function addLogMsg($relevance, $string)
	{
		if ($this->cFileOutput) {
			fwrite($this->cFilePointer, $this->_formatMsg($relevance, $string, FALSE));
		}

		$this->cLog[] = $this->_formatMsg($relevance, $string, TRUE);

		if ($relevance == FAULT) {
			foreach($this->getLog() as $line)
			{
				echo "$line<br />\n";
			}
			die($this->_formatMsg($relevance, $string, TRUE));
		}
		return ENOERROR;
	}


	public
	function getLog()
	{
		return $this->cLog;
	}


	public
	function debug($string)
	{
		if (Config("debug") == "on") {
			return $this->addLogMsg(DEBUG, $string);
		}
	}


	private
	function _formatMsg($relevance, $string, $htmlTags = TRUE)
	{
		$str = "";
		switch ($relevance)
		{
			case DEBUG:
				$str = "[DEBUG] $string\n";
				break;
			case SILENT:
				$str = "$string\n";
				break;
			case IMPORTANT:
				$str = "[IMPORTANT] $string\n";
				break;
			case WARNING:
				$str = ($htmlTags == TRUE) ?	"<strong>[WARNING]</strong> $string\n" :
								"[WARNING] $string\n";
				break;
			case FAULT:
				$str = ($htmlTags == TRUE) ?	"<strong>[FAULT] $string</strong>\n" :
								"[FAULT] $string\n";
				break;
			default:
				/* returns silent */
				$str = "[INVALID] $string\n";
				break;
		}

		return $str;
	}
}

?>
