<?php

interface iModules
{
	public function init($modName, $modInfo, $modInstance);
	public function register($modInstance, $eventType, $modName, $funcName);
	public function callEventFunctions($eventType, $moduleName = "");
}

class Modules implements iModules
{
	var $cSiteInstance;
	var $cModules = array();
	var $cModulesInfo = array();
	var $cEvents = array();

	function __construct($siteInstance)
	{
		$this->cSiteInstance = $siteInstance;

		$this->cSiteInstance->addLog(SILENT, "MODULE: Module interface initialized");
	}


	public
	function init($modName, $modInfo, $modInstance)
	{
		$this->cModulesInfo[$modName] = $modInfo;
		$this->cModules[$modName] = $modInstance;
	}


	public
	function register($modInstance, $eventType, $modName, $funcName)
	{
		switch($eventType)
		{
			case MOD_PAGEINIT:
			case MOD_OUTPUT:
				/* valid events */
				$this->cEvents[$eventType][$modName]['instance'] = $modInstance;
				$this->cEvents[$eventType][$modName]['funcname'] = $funcName;
				break;
			default:
				$this->cSiteInstance->addLog(IMPORTANT, "MODULE: invalid action type (" . getClassName($modInstance) . ")!");
				break;
		}

		$this->callEventFunctions(MOD_OUTPUT);
	}


	public
	function callEventFunctions($eventType, $moduleName = "")
	{
		/* if we have the module name call ONLY the function
		 * relative to that module/event, otherwise call all
		 * the functions relative to that event.
		 */


		if (!isset($this->cEvents[$eventType])) {
			return EGENERIC;
		}

		$eventArray = $this->cEvents[$eventType];
		
		if (strlen($moduleName) > 0) {
			if (	(isset($eventArray[$moduleName]['instance'])) &&
				(isset($eventArray[$moduleName]['funcname'])))
			{
				$instance = $eventArray[$moduleName]['instance'];
				$function = $eventArray[$moduleName]['funcname'];

				return $instance->$function(array());
			}
		} else {
			foreach($eventArray as $modName)
			{
				$instance = $modName['instance'];
				$function = $modName['funcname'];
			}
		}

		return $this->callFunction($instance, $function, $eventType);
	}


	private
	function callFunction($instance, $function, $eventType)
	{
		switch($eventType)
		{
			case MOD_PAGEINIT:
				return $instance->$function();
				break;
			case MOD_OUTPUT:
				$urlParam = $this->cSiteInstance->getGetArray(TRUE);
				return $instance->$function($urlParam);
				break;
			default:
				/* ignore it */
				return EGENERIC;
				break;
		}
	}
}
?>
