<?php

interface iPage {
	public function headAddLine($string);
	public function headAddStyle($stylesheet, $media = "all");
	public function headAddMeta($name, $content);
	public function headAddLink($rel, $href);
	public function headAddModule($moduleName);
	public function setCustomHtmlTag($htmlTag = "");
	public function addLine($string);
	public function addModule($moduleName);
	public function addBodyCustomParms($aParm = array());
	public function output($saveFile = "", $showOutput = FALSE);
}

class Page implements iPage
{
	private $cSiteInstance;

	private $cHtmlType;
	private $cHtmlDeclaration;
	private $cUseHead;
	private $cHead = array();
	private $cBodyCustomParms;
	private $cHtmlTag;

	private $cOutput = array();


	function __construct($siteInstance, $htmlType, $customType = "")
	{
		$this->cSiteInstance = $siteInstance;
		/* set some default value */
		$this->cBodyCustomParms = "";
		$this->setCustomHtmlTag("<html>");

		$this->cHtmlType = $htmlType;
		$this->cUseHead = TRUE;
		

		switch($htmlType)
		{
			case HTML401S:
				$doctype  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"';
				$doctype .= ' "http://www.w3.org/TR/html4/strict.dtd">';
				break;
			case HTML401T:
				$doctype  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"';
				$doctype .= ' "http://www.w3.org/TR/html4/loose.dtd">';
				break;
			case HTML401F:
				$doctype  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"';
				$doctype .= ' "http://www.w3.org/TR/html4/frameset.dtd">';
				break;
			case XHTML10S:
				$doctype  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"';
				$doctype .= ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
				break;
			case XHTML10T:
				$doctype  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"';
				$doctype .= ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
				break;
			case XHTML10F:
				$doctype  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"';
				$doctype .= ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">';
				break;
			case XHTML11:
				/* here we need a XML specification too */
				$doctype  = '<?xml version="1.0"?>' . "\n";
				$doctype .= '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "xhtml11.dtd">';
				break;
			case XCUSTOM:
				$this->cHtmlType = XCUSTOM;
				if (strlen($customType) == 0) {
					return ENOENOUGHPARMS;
				}
				$doctype = $customType;
				break;
			case NODOCTYPE:
				$this->cUseHead = FALSE;
			default:
				/* if unknown, switch to HTML 4.01 */
				$this->cHtmlType = HTML401;
				$doctype  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"';
				$doctype .= ' "http://www.w3.org/TR/html4/strict.dtd">';
				break;
		}

		$doctype .= "\n";

		$this->cHtmlDeclaration = $doctype;
	}


	public
	function headAddLine($string)
	{
		$this->cHead[] = $string;
	}



	/* these are just wrappers for headAddLine since these "head lines"
	 * are commonly used
	 */
	public
	function headAddStyle($stylesheet, $media = "all")
	{
		if (!isset($stylesheet)) {
			return ENOPARM;
		}
		$this->headAddLine('<link rel="stylesheet" href="'.$stylesheet.'" type="text/css" media="'.$media.'"');

		return ENOERROR;
	}


	public
	function headAddMeta($name, $content)
	{
		/* In this func, all the parms are mandatory. If you need something
		 * special, use a raw string passed to headAddLine()
		 */
		if (func_num_args() != 2) {
			return ENOENOUGHPARMS;
		}
		$this->headAddLine('<meta name="'.$name.'" content="'.$content.'" />');

		return ENOERROR;
	}


	public
	function headAddLink($rel, $href)
	{
		/* In this func, all the parms are mandatory. If you need something
		 * special, use a raw string passed to headAddLine()
		 */
		if (func_num_args() != 2) {
			return ENOENOUGHPARMS;
		}

		$this->addHeadLine('<link rel="'.$rel.'" href="'.$href.'" />');
	}


	public
	function headAddModule($moduleName)
	{
		$this->headAddLine($this->cSiteInstance->modCallEvent(MOD_OUTPUT, $moduleName));
	}


	/* If you want it empty, just set it to "" */
	public
	function setCustomHtmlTag($htmlTag = "")
	{
		$this->cHtmlTag = "$htmlTag\n";
		return ENOERROR;
	}


	public
	function addLine($string)
	{
		$this->cOutput[] = $string;
	}


	public
	function addModule($moduleName)
	{
		$this->addLine($this->cSiteInstance->modCallEvent(MOD_OUTPUT, $moduleName));
	}


	public
	function addBodyCustomParms($aParm = array())
	{
		if (!is_array($aParm))
			return AINVALIDPARMTYPE;


		foreach($aParm as $parm => $value)
		{
			if ((isset($value)) && (strlen($value) > 0)) {
				$this->cBodyCustomParms .= " $parm=\"$value\"";
			} else {
				$this->cBodyCustomParms .= " $parm";
			}
		}

		return ENOERROR;
	}


	/* begin page formatting functions */


	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */
	function addDiv($params)
	{
		if (func_num_args() != 1) {
			$this->cSiteInstance->addLog(SILENT, "No params passed to div creation");
			return ENOENOUGHPARMS;
		}

		if (!is_array($params)) {
			$this->cSiteInstance->addLog(SILENT, "Wrong data format");
			return EINVALIDPARMTYPE;
		}
	}
	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */
	/* TODO TODO TODO TODO TODO */

	/* end page formatting functions */


	public
	function output($saveFile = "", $showOutput = FALSE)
	{
		/* with $saveFile set, we have to store the producted
		 * page somewhere. In this case, normally we don't want
		 * to get our page showed up, but if you want this,
		 * just set $showOutput to TRUE. If $saveFile is
		 * empty, $showOutput won't be considered and output
		 * showed up.
		 */


		$fileMode = FALSE;
		$fp = "";

		if (strlen($saveFile) > 0) {
			$fileMode = TRUE;

			echo Config("site_path") . "/$saveFile";
			
			if (	(file_exists(Config("site_path") . "/$saveFile")) &&
				(!is_writable(Config("site_path") . "/$saveFile"))
				) {
				return ECANTOPEN;
			}

			$fp = fopen(Config("site_path") . "/$saveFile", "w");

			if (!isset($fp)) {
				return ECANTOPEN;
			}
		}

		if ($this->cUseHead == TRUE) {
			$this->_out($this->cHtmlDeclaration, $fp, $showOutput);
			$this->_out($this->cHtmlTag, $fp, $showOutput);
		

			/* print <head> */
			$this->_out("<head>\n", $fp, $showOutput);
			foreach($this->cHead as $line)
			{
				$this->_out("$line\n", $fp, $showOutput);
			}
			$this->_out("</head>\n", $fp, $showOutput);

			/* print <body> */
			if (!isset($cBodyCustomParms)) {
				$cBodyCustomParms = "";
			}
			$this->_out("<body$cBodyCustomParms>\n", $fp, $showOutput);
		}

		/* print the page core out */
		foreach($this->cOutput as $line)
		{
			$this->_out("$line\n", $fp, $showOutput);
		}


		if ($this->cUseHead = TRUE) {
			$this->_out("</body>\n", $fp, $showOutput);

			$this->_out("</html>\n", $fp, $showOutput);
		}


		if ($fileMode == TRUE) {
			fclose($fp);
		}


		return ENOERROR;
	}


	/* private functions */

	private
	function _out($string, $fileHandle = "", $outputAlways = FALSE)
	{
		/* if $fileHandle is a file pointer, necessarly won't be a string too
		 * and since we want to save the output on a file just when this fileHandle
		 * is correctly opened, just check for this
		 */
		
		if (is_string($fileHandle)) {
			echo $string;
		} else {
			if (!fwrite($fileHandle, $string, strlen($string))) {
				return ECANTOPEN;
			}

			if ($outputAlways == TRUE) {
				echo $string;
			}
		}

		return ENOERROR;
	}
}

?>
